@@ -1,3 +1,4 @@
+import asyncio
 import base64
 import hashlib
 import hmac
@@ -8,13 +9,17 @@ import threading
 from datetime import datetime
 from queue import Queue
 from time import mktime
-from typing import Any, Dict, Generator, Iterator, List, Mapping, Optional, Type
+from typing import Any, Dict, Generator, Iterator, List, Mapping, Optional, Type, AsyncIterator, AsyncGenerator, Union
 from urllib.parse import urlencode, urlparse, urlunparse
 from wsgiref.handlers import format_date_time
 
+import httpx
+import websocket
+
 from sparkai.core.callbacks import (
-    CallbackManagerForLLMRun, BaseCallbackHandler,
+    CallbackManagerForLLMRun, BaseCallbackHandler, AsyncCallbackManagerForLLMRun, AsyncCallbackHandler,
 )
+from sparkai.core.callbacks.manager import ahandle_event
 from sparkai.core.language_models.chat_models import (
     BaseChatModel,
     generate_from_stream,
@@ -24,6 +29,8 @@ from sparkai.core.messages import (
     AIMessageChunk,
     BaseMessage,
     BaseMessageChunk,
+    ImageChatMessage,
+    ImageChatMessageChunk,
     ChatMessage,
     ChatMessageChunk,
     HumanMessage,
@@ -36,26 +43,44 @@ from sparkai.core.messages import (
 from sparkai.core.outputs import (
     ChatGeneration,
     ChatGenerationChunk,
-    ChatResult,
+    ChatResult, GenerationChunk,
 )
 from sparkai.core.pydantic_v1 import Field, root_validator
 from sparkai.core.utils import (
     get_from_dict_or_env,
     get_pydantic_field_names,
 )
+from sparkai.v2.client.common.consts import IFLYTEK
+from sparkai.version import __version__
 
-logger = logging.getLogger(__name__)
+from sparkai.log.logger import logger
+
+DefaultDomain = "generalv3.5"
 
 
-def _convert_message_to_dict(message: BaseMessage) -> dict:
+def convert_message_to_dict(messages: List[BaseMessage]) -> List[dict]:
+    new = []
+    for i, message in enumerate(messages):
+        new.append(_convert_message_to_dict(i, message))
+    return new
+
+
+def _convert_message_to_dict(index: int, message: BaseMessage) -> dict:
     if isinstance(message, ChatMessage):
-        message_dict = {"role": "user", "content": message.content}
+        if message.role == "system" and index != 0:
+            message.role = "user"
+        message_dict = {"role": message.role, "content": message.content}
+    elif isinstance(message, ImageChatMessage):
+        message_dict = {"role": "user", "content": message.content, "content_type": message.content_type}
     elif isinstance(message, HumanMessage):
         message_dict = {"role": "user", "content": message.content}
     elif isinstance(message, AIMessage):
         message_dict = {"role": "assistant", "content": message.content}
     elif isinstance(message, SystemMessage):
-        message_dict = {"role": "system", "content": message.content}
+        role = "system"
+        if index != 0:
+            role = "user"
+        message_dict = {"role": role, "content": message.content}
     else:
         raise ValueError(f"Got unknown type {message}")
 
@@ -63,8 +88,8 @@ def _convert_message_to_dict(message: BaseMessage) -> dict:
 
 
 def _convert_dict_to_message(_dict: Mapping[str, Any]) -> BaseMessage:
-    msg_role = _dict["role"]
-    msg_content = _dict["content"]
+    msg_role = _dict.get("role")
+    msg_content = _dict.get("content")
     if "function_call" in _dict:
         msg_role = "function_call"
     if msg_role == "user":
@@ -75,7 +100,8 @@ def _convert_dict_to_message(_dict: Mapping[str, Any]) -> BaseMessage:
     elif msg_role == "system":
         return SystemMessage(content=msg_content)
     elif msg_role == "function_call" or "function_call" in _dict:
-        return FunctionCallMessage( content=msg_content, function_call=_dict["function_call"])
+        return FunctionCallMessage(content=msg_content, function_call=_dict["function_call"])
+
     else:
         return ChatMessage(content=msg_content, role=msg_role)
 
@@ -94,7 +120,7 @@ def _convert_delta_to_message_chunk(
     elif msg_role == "assistant" or default_class == AIMessageChunk or default_class == FunctionMessageChunk:
         return AIMessageChunk(content=msg_content)
     elif msg_role == "function_call" or "function_call" in _dict:
-        return FunctionCallMessageChunk( content=msg_content, function_call=_dict["function_call"])
+        return FunctionCallMessageChunk(content=msg_content, function_call=_dict["function_call"])
     elif msg_role or default_class == ChatMessageChunk:
         return ChatMessageChunk(content=msg_content, role=msg_role)
     else:
@@ -140,11 +166,14 @@ class ChatSparkLLM(BaseChatModel):
     spark_api_secret: Optional[str] = None
     spark_api_url: Optional[str] = None
     spark_llm_domain: Optional[str] = None
-    spark_user_id: str = "lc_user"
+    user_agent: Optional[str] = None
+    spark_user_id: str = "spark_sdk_user"
     streaming: bool = False
     request_timeout: int = 30
     temperature: float = 0.5
     top_k: int = 4
+    max_tokens: int = 2048
+
     model_kwargs: Dict[str, Any] = Field(default_factory=dict)
 
     @root_validator(pre=True)
@@ -206,6 +235,7 @@ class ChatSparkLLM(BaseChatModel):
         # put extra params into model_kwargs
         values["model_kwargs"]["temperature"] = values["temperature"] or cls.temperature
         values["model_kwargs"]["top_k"] = values["top_k"] or cls.top_k
+        values["model_kwargs"]["max_tokens"] = values["max_tokens"] or cls.max_tokens
 
         values["client"] = _SparkLLMClient(
             app_id=values["spark_app_id"],
@@ -214,9 +244,54 @@ class ChatSparkLLM(BaseChatModel):
             api_url=values["spark_api_url"],
             spark_domain=values["spark_llm_domain"],
             model_kwargs=values["model_kwargs"],
+            user_agent=values["user_agent"],
         )
         return values
 
+    async def _astream(
+            self,
+            messages: List[BaseMessage],
+            stop: Optional[List[str]] = None,
+            run_manager: Optional[AsyncCallbackManagerForLLMRun] = None,
+            **kwargs: Any,
+    ) -> AsyncIterator[ChatGenerationChunk]:
+        default_chunk_class = AIMessageChunk
+        function_definition = []
+        if "function_definition" in kwargs:
+            function_definition = kwargs['function_definition']
+
+        llm_output = {}
+        if "llm_output" in kwargs:
+            llm_output = kwargs.get("llm_output")
+        else:
+            kwargs["llm_output"] = llm_output
+        converted_messages = convert_message_to_dict(messages)
+
+        self.client.arun(
+            converted_messages,
+            self.spark_user_id,
+            self.model_kwargs,
+            self.streaming,
+            function_definition
+        )
+        final_frame = False
+        async for content in self.client.a_subscribe(timeout=self.request_timeout):
+            if "usage" in content:
+                final_frame = True
+                llm_output["token_usage"] = content["usage"]
+            if "data" not in content and not final_frame:
+                continue
+            if not final_frame:
+                delta = content["data"]
+                chunk = _convert_delta_to_message_chunk(delta, default_chunk_class)
+            else:
+                delta = {}
+            if run_manager:
+                await run_manager.on_llm_new_token(str(chunk.content), llm_output=llm_output, data=delta,
+                                                   final=final_frame)
+
+            yield ChatGenerationChunk(message=chunk)
+
     def _stream(
             self,
             messages: List[BaseMessage],
@@ -229,24 +304,36 @@ class ChatSparkLLM(BaseChatModel):
         if "function_definition" in kwargs:
             function_definition = kwargs['function_definition']
 
+        llm_output = {}
+        if "llm_output" in kwargs:
+            llm_output = kwargs.get("llm_output")
+        else:
+            kwargs["llm_output"] = llm_output
+
+        converted_messages = convert_message_to_dict(messages)
+
         self.client.arun(
-            [_convert_message_to_dict(m) for m in messages],
+            converted_messages,
             self.spark_user_id,
             self.model_kwargs,
             self.streaming,
             function_definition
         )
-        llm_output = kwargs.get("llm_output")
+        final_frame = False
         for content in self.client.subscribe(timeout=self.request_timeout):
             if "usage" in content:
+                final_frame = True
                 llm_output["token_usage"] = content["usage"]
-            if "data" not in content:
+            if "data" not in content and not final_frame:
                 continue
-            delta = content["data"]
-            chunk = _convert_delta_to_message_chunk(delta, default_chunk_class)
-            yield ChatGenerationChunk(message=chunk)
+            if not final_frame:
+                delta = content["data"]
+                chunk = _convert_delta_to_message_chunk(delta, default_chunk_class)
+            else:
+                delta =  {}
             if run_manager:
-                run_manager.on_llm_new_token(str(chunk.content))
+                run_manager.on_llm_new_token(str(chunk.content), llm_output=llm_output, data=delta, final=final_frame)
+            yield ChatGenerationChunk(message=chunk)
 
     def _generate(
             self,
@@ -270,9 +357,9 @@ class ChatSparkLLM(BaseChatModel):
             )
             return generate_from_stream(stream_iter, llm_output)
 
-
+        converted_messages = convert_message_to_dict(messages)
         self.client.arun(
-            [_convert_message_to_dict(m) for m in messages],
+            converted_messages,
             self.spark_user_id,
             self.model_kwargs,
             False,
@@ -285,6 +372,9 @@ class ChatSparkLLM(BaseChatModel):
             if "data" not in content:
                 continue
             completion = content["data"]
+        if not completion:
+            logger.error("No completion Generate")
+            return ChatResult(generations=[], llm_output={})
         message = _convert_dict_to_message(completion)
         generations = [ChatGeneration(message=message)]
         return ChatResult(generations=generations, llm_output=llm_output)
@@ -294,6 +384,13 @@ class ChatSparkLLM(BaseChatModel):
         return "spark-llm-chat"
 
 
+def prepare_user_agent(user_agent):
+    extra_user_agent = ""
+    if user_agent:
+        extra_user_agent = user_agent
+    return {"User-Agent": "SparkAISdk/python-v%s %s" % (__version__, extra_user_agent)}
+
+
 class _SparkLLMClient:
     """
     Use websocket-client to call the SparkLLM interface provided by Xfyun,
@@ -308,45 +405,74 @@ class _SparkLLMClient:
             api_url: Optional[str] = None,
             spark_domain: Optional[str] = None,
             model_kwargs: Optional[dict] = None,
+            user_agent: Optional[str] = None,
+            provider=IFLYTEK,
+            is_ws=True
+
     ):
-        try:
-            import websocket
+        self.is_ws = is_ws
+        if self.is_ws:
+            self.client = websocket
+        else:
+            self.client = httpx.Client()
 
-            self.websocket_client = websocket
-        except ImportError:
-            raise ImportError(
-                "Could not import websocket client python package. "
-                "Please install it with `pip install websocket-client`."
-            )
+        self.spark_domain = spark_domain or DefaultDomain
+
+        if api_url is None or api_url == "":
+            self.api_url = self._adjust_api_by_domain("", spark_domain)
+        else:
+            self.api_url = self._adjust_api_by_domain(api_url, spark_domain)
 
-        self.api_url = (
-            "wss://spark-api.xf-yun.com/v3.1/chat" if not api_url else api_url
-        )
         self.app_id = app_id
-        self.ws_url = _SparkLLMClient._create_url(
-            self.api_url,
-            api_key,
-            api_secret,
-        )
+        self.api_key = api_key
+        self.api_secret = api_secret
         self.model_kwargs = model_kwargs
-        self.spark_domain = spark_domain or "generalv3"
         self.queue: Queue[Dict] = Queue()
         self.blocking_message = {"content": "", "role": "assistant"}
+        self.api_key = api_key
+        self.api_secret = api_secret
+        self.user_agent = prepare_user_agent(user_agent)
+
+    def _adjust_api_by_domain(self, url: str, domain: str) -> str:
+        """
+        Adjust the api base according the domain provided
+        :param domain:
+        :return:
+        """
+        host_map = {
+            "generalv3.5": "wss://spark-api.xf-yun.com/v3.5/chat",
+            "generalv3": "wss://spark-api.xf-yun.com/v3.1/chat",
+            "generalv2": "wss://spark-api.xf-yun.com/v2.1/chat",
+            "general": "wss://spark-api.xf-yun.com/v1.1/chat",
+            "image": "wss://spark-api.cn-huabei-1.xf-yun.com/v2.1/image",
+        }
+        if domain == "":
+            domain = DefaultDomain
+
+        if domain not in host_map:
+            domain = DefaultDomain
+            logger.warning("not find the  domain, using default domain: %s", domain)
+            return url
+
+        if url != "" and url != host_map[domain]:
+            logger.warning("specified host not match the domain default host, using default domain: %s", domain)
+
+        return host_map[domain]
 
     @staticmethod
-    def _create_url(api_url: str, api_key: str, api_secret: str) -> str:
+    def _create_url(api_url: str, api_key: str, api_secret: str, method="GET") -> str:
         """
         Generate a request url with an api key and an api secret.
         """
         # generate timestamp by RFC1123
         date = format_date_time(mktime(datetime.now().timetuple()))
-
+        encrypt_method = "hmac-sha256"
         # urlparse
         parsed_url = urlparse(api_url)
         host = parsed_url.netloc
         path = parsed_url.path
 
-        signature_origin = f"host: {host}\ndate: {date}\nGET {path} HTTP/1.1"
+        signature_origin = f"host: {host}\ndate: {date}\n{method} {path} HTTP/1.1"
 
         # encrypt using hmac-sha256
         signature_sha = hmac.new(
@@ -357,7 +483,7 @@ class _SparkLLMClient:
 
         signature_sha_base64 = base64.b64encode(signature_sha).decode(encoding="utf-8")
 
-        authorization_origin = f'api_key="{api_key}", algorithm="hmac-sha256", \
+        authorization_origin = f'api_key="{api_key}", algorithm="{encrypt_method}", \
         headers="host date request-line", signature="{signature_sha_base64}"'
         authorization = base64.b64encode(authorization_origin.encode("utf-8")).decode(
             encoding="utf-8"
@@ -386,20 +512,31 @@ class _SparkLLMClient:
             streaming: bool = False,
             function_definition: List[Dict] = []
     ) -> None:
-        self.websocket_client.enableTrace(False)
-        ws = self.websocket_client.WebSocketApp(
-            self.ws_url,
-            on_message=self.on_message,
-            on_error=self.on_error,
-            on_close=self.on_close,
-            on_open=self.on_open,
-        )
-        ws.function_definition = function_definition
-        ws.messages = messages
-        ws.user_id = user_id
-        ws.model_kwargs = self.model_kwargs if model_kwargs is None else model_kwargs
-        ws.streaming = streaming
-        ws.run_forever()
+        if self.is_ws:
+            self.client.enableTrace(False)
+            ws = self.client.WebSocketApp(
+                _SparkLLMClient._create_url(
+                    self.api_url,
+                    self.api_key,
+                    self.api_secret,
+
+                ),
+                on_message=self.on_message,
+                on_error=self.on_error,
+                on_close=self.on_close,
+                on_open=self.on_open,
+                header=self.user_agent
+            )
+            ws.function_definition = function_definition
+            ws.messages = messages
+            ws.user_id = user_id
+            ws.model_kwargs = self.model_kwargs if model_kwargs is None else model_kwargs
+            ws.streaming = streaming
+            ws.run_forever()
+        else:
+            # HTTP logic
+            result = self.request(messages)
+            self.queue.put()
 
     def arun(
             self,
@@ -409,7 +546,7 @@ class _SparkLLMClient:
             streaming: bool = False,
             function_definition: List[Dict] = [],
     ) -> threading.Thread:
-        ws_thread = threading.Thread(
+        req_thread = threading.Thread(
             target=self.run,
             args=(
                 messages,
@@ -419,8 +556,8 @@ class _SparkLLMClient:
                 function_definition,
             ),
         )
-        ws_thread.start()
-        return ws_thread
+        req_thread.start()
+        return req_thread
 
     def on_error(self, ws: Any, error: Optional[Any]) -> None:
         self.queue.put({"error": error})
@@ -450,9 +587,10 @@ class _SparkLLMClient:
     def on_message(self, ws: Any, message: str) -> None:
         data = json.loads(message)
         code = data["header"]["code"]
+        logger.debug(f"sid: {data['header']['sid']}, code: {code}")
         if code != 0:
             self.queue.put(
-                {"error": f"Code: {code}, Error: {data['header']['message']}"}
+                {"error": f"Error Code: {code}, Error: {data['header']['message']}"}
             )
             ws.close()
         else:
@@ -497,16 +635,26 @@ class _SparkLLMClient:
         logger.debug(f"Spark Request Parameters: {data}")
         return data
 
-    def subscribe(self, timeout: Optional[int] = 30) -> Generator[Dict, None, None]:
+    async def a_subscribe(self, timeout: Optional[int] = 30) -> AsyncGenerator[Dict, None]:
+        err_cnt = 0
         while True:
             try:
                 content = self.queue.get(timeout=timeout)
             except queue.Empty as _:
-                raise TimeoutError(
+                e = TimeoutError(
                     f"SparkLLMClient wait LLM api response timeout {timeout} seconds"
                 )
+                logger.error(e)
+                err_cnt += 1
+                if err_cnt >= 4:
+                    raise e
+                else:
+                    continue
             if "error" in content:
-                raise ConnectionError(content["error"])
+                e = ConnectionError(content["error"])
+                err_cnt += 1
+                raise e
+
             if "usage" in content:
                 yield content
                 continue
@@ -516,6 +664,43 @@ class _SparkLLMClient:
                 break
             yield content
 
+    def subscribe(self, timeout: Optional[int] = 30) -> Generator[Dict, None, None]:
+        err_cnt = 0
+        while True:
+            try:
+                content = self.queue.get(timeout=timeout)
+            except queue.Empty as _:
+                e = TimeoutError(
+                    f"SparkLLMClient wait LLM api response timeout {timeout} seconds"
+                )
+                logger.error(e)
+                err_cnt += 1
+                if err_cnt >= 4:
+                    raise e
+                else:
+                    continue
+            if "error" in content:
+                e = ConnectionError(content["error"])
+                err_cnt += 1
+                raise e
+
+            if "usage" in content:
+                yield content
+                continue
+            # continue
+            if "done" in content:
+                break
+            if "data" not in content:
+                break
+            yield content
+
+    # def request(self, messages):
+    #     resp = self.client.get('')
+    #     result = resp.json()
+    #     # print(result)
+    #     assert result["code"] == 300
+    #     return result
+
 
 class ChunkPrintHandler(BaseCallbackHandler):
     """Callback Handler that prints to std out."""
@@ -529,3 +714,31 @@ class ChunkPrintHandler(BaseCallbackHandler):
                          chunk: None,
                          **kwargs: Any, ):
         print(token)
+        print(kwargs)
+
+
+class AsyncChunkPrintHandler(AsyncCallbackHandler):
+    """Callback Handler that prints to std out."""
+
+    def __init__(self, color: Optional[str] = None) -> None:
+        """Initialize callback handler."""
+        super().__init__()
+        self.color = color
+
+
+    async def on_llm_new_token(
+        self,
+        token: str,
+        *,
+        chunk: Optional[Union[GenerationChunk, ChatGenerationChunk]] = None,
+        **kwargs: Any,
+    ) -> None:
+        """Run when LLM generates a new token.
+
+        Args:
+            token (str): The new token.
+        """
+        print(token)
+        print(kwargs)
+
+
